/*
 * Copyright (c) 2004 Marc Balmer <marc@msys.ch>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <sys/stat.h>
#include <signal.h>
#include <unistd.h>
#include <ctype.h>
#include <netdb.h>
#include <err.h>
#include <syslog.h>

#include "smtp-vilter.h"

#ifdef LINUX
#include <sys/param.h>
#include "strlfunc.h"
#include "sys/queue.h"
#else
#include <sys/queue.h>
#include <sys/syslimits.h>
#endif

#define	REGEXP_CONF	"/etc/smtp-vilter/regexp.conf"

#define TIMEOUT 	60
#define MAXBUF		1024
#define MAXSTR		64
#define MAXTRIES	256

SLIST_HEAD(patshead, pattern) pats;

struct pattern {
	char *pat;
	regex_t preg;
	SLIST_ENTRY(pattern) pats;
};

int
vilter_init(char *cfgfile)
{
	FILE *fp;
	char field[MAXSTR];
	char value[MAXSTR];

	/* Default values */
			
	if (cfgfile == NULL)
		cfgfile = REGEXP_CONF;
		
	if ((fp = fopen(cfgfile, "r")) != NULL) {
		if (verbose)
			warnx("regexp: using configuration from file %s", cfgfile);
			
		while (!read_conf(fp, field, sizeof(field), value, sizeof(value))) {
			if (verbose)
				warnx("regexp: unknown command %s=%s", field, value);
		}
		fclose(fp);
	} else if (verbose)
		warnx("regexp: configuration file %s for regexp backend not found, using default values", cfgfile);
	
	return 0;
}


int
vilter_scan(struct be_data *priv, char *fn, char *chroot, char *reason, int rlen)
{
	int fd, datfd;
	int try;
	int nread;
	FILE *fp;
	unsigned short port;
	struct sockaddr_in server_sockaddr, data_sockaddr;
	struct timeval tv, *t;
	struct hostent *hostent;
	char buf[MAXBUF];
	char *p, *q;
		
	return SCAN_ERROR;
}

char *
vilter_name(void)
{
	return "Regular Expression Filter (regexp)";
}

int
vilter_type(void)
{
	return SCAN_UNWANTED;
}

void
vilter_exit(void)
{
}
